@@ -40,10 +40,8 @@
 #include "storage.h"
 #include "idmap.h"
 
-#define DATA_CONNECTION_MANAGER_INTERFACE "org.ofono.DataConnectionManager"
-#define DATA_CONTEXT_INTERFACE "org.ofono.PrimaryDataContext"
-
 #define GPRS_FLAG_ATTACHING 0x1
+#define GPRS_FLAG_RECHECK 0x2
 
 #define SETTINGS_STORE "gprs"
 #define SETTINGS_GROUP "Settings"
@@ -67,9 +65,6 @@ struct ofono_gprs {
 	ofono_bool_t roaming_allowed;
 	ofono_bool_t powered;
 	int status;
-	int location;
-	int cellid;
-	int technology;
 	int flags;
 	struct idmap *pid_map;
 	unsigned int last_context_id;
@@ -261,7 +256,7 @@ static void pri_context_signal_settings(struct pri_context *ctx)
 	DBusMessageIter iter;
 	const char *prop = "Settings";
 
-	signal = dbus_message_new_signal(path, DATA_CONTEXT_INTERFACE,
+	signal = dbus_message_new_signal(path, OFONO_DATA_CONTEXT_INTERFACE,
 						"PropertyChanged");
 
 	if (!signal)
@@ -416,7 +411,7 @@ static void pri_activate_callback(const struct ofono_error *error,
 	DBG("%p %s", ctx, interface);
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Activating context failed with error: %s",
+		DBG("Activating context failed with error: %s",
 				telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&gc->pending,
 					__ofono_error_failed(gc->pending));
@@ -440,7 +435,7 @@ static void pri_activate_callback(const struct ofono_error *error,
 
 	value = ctx->active;
 	ofono_dbus_signal_property_changed(conn, ctx->path,
-						DATA_CONTEXT_INTERFACE,
+						OFONO_DATA_CONTEXT_INTERFACE,
 						"Active", DBUS_TYPE_BOOLEAN,
 						&value);
 }
@@ -453,7 +448,7 @@ static void pri_deactivate_callback(const struct ofono_error *error, void *data)
 	dbus_bool_t value;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Deactivating context failed with error: %s",
+		DBG("Deactivating context failed with error: %s",
 				telephony_error_to_str(error));
 		__ofono_dbus_pending_reply(&gc->pending,
 					__ofono_error_failed(gc->pending));
@@ -471,7 +466,7 @@ static void pri_deactivate_callback(const struct ofono_error *error, void *data)
 
 	value = ctx->active;
 	ofono_dbus_signal_property_changed(conn, ctx->path,
-						DATA_CONTEXT_INTERFACE,
+						OFONO_DATA_CONTEXT_INTERFACE,
 						"Active", DBUS_TYPE_BOOLEAN,
 						&value);
 }
@@ -501,7 +496,7 @@ static DBusMessage *pri_set_apn(struct pri_context *ctx, DBusConnection *conn,
 	g_dbus_send_reply(conn, msg, DBUS_TYPE_INVALID);
 
 	ofono_dbus_signal_property_changed(conn, ctx->path,
-						DATA_CONTEXT_INTERFACE,
+						OFONO_DATA_CONTEXT_INTERFACE,
 						"AccessPointName",
 						DBUS_TYPE_STRING, &apn);
 
@@ -531,7 +526,7 @@ static DBusMessage *pri_set_username(struct pri_context *ctx,
 	g_dbus_send_reply(conn, msg, DBUS_TYPE_INVALID);
 
 	ofono_dbus_signal_property_changed(conn, ctx->path,
-						DATA_CONTEXT_INTERFACE,
+						OFONO_DATA_CONTEXT_INTERFACE,
 						"Username",
 						DBUS_TYPE_STRING, &username);
 
@@ -561,7 +556,7 @@ static DBusMessage *pri_set_password(struct pri_context *ctx,
 	g_dbus_send_reply(conn, msg, DBUS_TYPE_INVALID);
 
 	ofono_dbus_signal_property_changed(conn, ctx->path,
-						DATA_CONTEXT_INTERFACE,
+						OFONO_DATA_CONTEXT_INTERFACE,
 						"Password",
 						DBUS_TYPE_STRING, &password);
 
@@ -592,8 +587,8 @@ static DBusMessage *pri_set_type(struct pri_context *ctx, DBusConnection *conn,
 	g_dbus_send_reply(conn, msg, DBUS_TYPE_INVALID);
 
 	ofono_dbus_signal_property_changed(conn, ctx->path,
-						DATA_CONTEXT_INTERFACE, "Type",
-						DBUS_TYPE_STRING, &type);
+						OFONO_DATA_CONTEXT_INTERFACE,
+						"Type", DBUS_TYPE_STRING, &type);
 
 	return NULL;
 }
@@ -619,8 +614,8 @@ static DBusMessage *pri_set_name(struct pri_context *ctx, DBusConnection *conn,
 	g_dbus_send_reply(conn, msg, DBUS_TYPE_INVALID);
 
 	ofono_dbus_signal_property_changed(conn, ctx->path,
-						DATA_CONTEXT_INTERFACE, "Name",
-						DBUS_TYPE_STRING, &name);
+						OFONO_DATA_CONTEXT_INTERFACE,
+						"Name", DBUS_TYPE_STRING, &name);
 
 	return NULL;
 }
@@ -800,7 +795,7 @@ static gboolean context_dbus_register(struct pri_context *ctx)
 
 	snprintf(path, sizeof(path), "%s/primarycontext%u", basepath, ctx->id);
 
-	if (!g_dbus_register_interface(conn, path, DATA_CONTEXT_INTERFACE,
+	if (!g_dbus_register_interface(conn, path, OFONO_DATA_CONTEXT_INTERFACE,
 					context_methods, context_signals,
 					NULL, ctx, pri_context_destroy)) {
 		ofono_error("Could not register PrimaryContext %s", path);
@@ -825,7 +820,7 @@ static gboolean context_dbus_unregister(struct pri_context *ctx)
 	idmap_put(ctx->gprs->pid_map, ctx->id);
 
 	return g_dbus_unregister_interface(conn, path,
-						DATA_CONTEXT_INTERFACE);
+						OFONO_DATA_CONTEXT_INTERFACE);
 }
 
 static char **gprs_contexts_path_list(GSList *context_list)
@@ -846,79 +841,6 @@ static char **gprs_contexts_path_list(GSList *context_list)
 	return objlist;
 }
 
-static void set_registration_status(struct ofono_gprs *gprs, int status)
-{
-	const char *str_status = registration_status_to_string(status);
-	const char *path = __ofono_atom_get_path(gprs->atom);
-	DBusConnection *conn = ofono_dbus_get_connection();
-
-	if (gprs->status == status)
-		return;
-
-	gprs->status = status;
-
-	ofono_dbus_signal_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
-					"Status", DBUS_TYPE_STRING,
-					&str_status);
-}
-
-static void set_registration_location(struct ofono_gprs *gprs,
-					int lac)
-{
-	DBusConnection *conn = ofono_dbus_get_connection();
-	const char *path = __ofono_atom_get_path(gprs->atom);
-	dbus_uint16_t dbus_lac = lac;
-
-	if (lac > 0xffff)
-		return;
-
-	gprs->location = lac;
-
-	if (gprs->location == -1)
-		return;
-
-	ofono_dbus_signal_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
-					"LocationAreaCode",
-					DBUS_TYPE_UINT16, &dbus_lac);
-}
-
-static void set_registration_cellid(struct ofono_gprs *gprs, int ci)
-{
-	DBusConnection *conn = ofono_dbus_get_connection();
-	const char *path = __ofono_atom_get_path(gprs->atom);
-	dbus_uint32_t dbus_ci = ci;
-
-	gprs->cellid = ci;
-
-	if (gprs->cellid == -1)
-		return;
-
-	ofono_dbus_signal_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
-					"CellId", DBUS_TYPE_UINT32,
-					&dbus_ci);
-}
-
-static void set_registration_technology(struct ofono_gprs *gprs,
-					int tech)
-{
-	const char *tech_str = registration_tech_to_string(tech);
-	DBusConnection *conn = ofono_dbus_get_connection();
-	const char *path = __ofono_atom_get_path(gprs->atom);
-
-	gprs->technology = tech;
-
-	if (gprs->technology == -1)
-		return;
-
-	ofono_dbus_signal_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
-					"Technology", DBUS_TYPE_STRING,
-					&tech_str);
-}
-
 static void gprs_attached_update(struct ofono_gprs *gprs)
 {
 	DBusConnection *conn = ofono_dbus_get_connection();
@@ -953,7 +875,7 @@ static void gprs_attached_update(struct ofono_gprs *gprs)
 
 			value = FALSE;
 			ofono_dbus_signal_property_changed(conn, ctx->path,
-						DATA_CONTEXT_INTERFACE,
+						OFONO_DATA_CONTEXT_INTERFACE,
 						"Active", DBUS_TYPE_BOOLEAN,
 						&value);
 		}
@@ -962,38 +884,45 @@ static void gprs_attached_update(struct ofono_gprs *gprs)
 	path = __ofono_atom_get_path(gprs->atom);
 	value = attached;
 	ofono_dbus_signal_property_changed(conn, path,
-				DATA_CONNECTION_MANAGER_INTERFACE,
+				OFONO_DATA_CONNECTION_MANAGER_INTERFACE,
 				"Attached", DBUS_TYPE_BOOLEAN, &value);
 }
 
 static void registration_status_cb(const struct ofono_error *error,
-					int status, int lac, int ci, int tech,
-					void *data)
+					int status, void *data)
 {
 	struct ofono_gprs *gprs = data;
 
 	if (error->type == OFONO_ERROR_TYPE_NO_ERROR)
-		ofono_gprs_status_notify(gprs, status, lac, ci, tech);
+		ofono_gprs_status_notify(gprs, status);
 
-	gprs->flags &= ~GPRS_FLAG_ATTACHING;
+	if (gprs->flags & GPRS_FLAG_RECHECK) {
+		gprs->flags &= ~GPRS_FLAG_RECHECK;
+		gprs_netreg_update(gprs);
+	}
 }
 
 static void gprs_attach_callback(const struct ofono_error *error, void *data)
 {
 	struct ofono_gprs *gprs = data;
 
-	if (error->type == OFONO_ERROR_TYPE_NO_ERROR) {
-		gprs->driver_attached = !gprs->driver_attached;
-		gprs_attached_update(gprs);
-	}
+	gprs->flags &= ~GPRS_FLAG_ATTACHING;
 
-	if (gprs->driver->registration_status) {
-		gprs->driver->registration_status(gprs, registration_status_cb,
-							gprs);
+	if (error->type != OFONO_ERROR_TYPE_NO_ERROR)
+		gprs->driver_attached = !gprs->driver_attached;
+
+	if (gprs->driver->attached_status) {
+		gprs->driver->attached_status(gprs, registration_status_cb,
+						gprs);
 		return;
 	}
 
-	gprs->flags &= ~GPRS_FLAG_ATTACHING;
+	gprs_attached_update(gprs);
+
+	if (gprs->flags & GPRS_FLAG_RECHECK) {
+		gprs->flags &= ~GPRS_FLAG_RECHECK;
+		gprs_netreg_update(gprs);
+	}
 }
 
 static void gprs_netreg_update(struct ofono_gprs *gprs)
@@ -1010,12 +939,15 @@ static void gprs_netreg_update(struct ofono_gprs *gprs)
 	if (gprs->driver_attached == attach)
 		return;
 
-	if (gprs->flags & GPRS_FLAG_ATTACHING)
+	if (gprs->flags & GPRS_FLAG_ATTACHING) {
+		gprs->flags |= GPRS_FLAG_RECHECK;
 		return;
+	}
 
 	gprs->flags |= GPRS_FLAG_ATTACHING;
 
 	gprs->driver->set_attached(gprs, attach, gprs_attach_callback, gprs);
+	gprs->driver_attached = attach;
 }
 
 static void netreg_status_changed(int status, int lac, int ci, int tech,
@@ -1043,7 +975,6 @@ static DBusMessage *gprs_get_properties(DBusConnection *conn,
 	DBusMessageIter dict;
 	char **objpath_list;
 	dbus_bool_t value;
-	const char *status = registration_status_to_string(gprs->status);
 
 	reply = dbus_message_new_method_return(msg);
 	if (!reply)
@@ -1074,28 +1005,6 @@ static DBusMessage *gprs_get_properties(DBusConnection *conn,
 	value = gprs->powered;
 	ofono_dbus_dict_append(&dict, "Powered", DBUS_TYPE_BOOLEAN, &value);
 
-	ofono_dbus_dict_append(&dict, "Status", DBUS_TYPE_STRING, &status);
-
-	if (gprs->location != -1) {
-		dbus_uint16_t location = gprs->location;
-		ofono_dbus_dict_append(&dict, "LocationAreaCode",
-					DBUS_TYPE_UINT16, &location);
-	}
-
-	if (gprs->cellid != -1) {
-		dbus_uint32_t cellid = gprs->cellid;
-		ofono_dbus_dict_append(&dict, "CellId",
-					DBUS_TYPE_UINT32, &cellid);
-	}
-
-	if (gprs->technology != -1) {
-		const char *technology =
-			registration_tech_to_string(gprs->technology);
-
-		ofono_dbus_dict_append(&dict, "Technology", DBUS_TYPE_STRING,
-					&technology);
-	}
-
 	dbus_message_iter_close_container(&iter, &dict);
 
 	return reply;
@@ -1176,7 +1085,7 @@ static DBusMessage *gprs_set_property(DBusConnection *conn,
 
 	path = __ofono_atom_get_path(gprs->atom);
 	ofono_dbus_signal_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
+					OFONO_DATA_CONNECTION_MANAGER_INTERFACE,
 					property, DBUS_TYPE_BOOLEAN, &value);
 
 	return dbus_message_new_method_return(msg);
@@ -1223,7 +1132,7 @@ static DBusMessage *gprs_create_context(DBusConnection *conn,
 		return __ofono_error_failed(msg);
 	}
 
-	ofono_debug("Registering new context");
+	DBG("Registering new context");
 
 	if (!context_dbus_register(context)) {
 		ofono_error("Unable to register primary context");
@@ -1254,7 +1163,7 @@ static DBusMessage *gprs_create_context(DBusConnection *conn,
 	if (objpath_list) {
 		path = __ofono_atom_get_path(gprs->atom);
 		ofono_dbus_signal_array_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
+					OFONO_DATA_CONNECTION_MANAGER_INTERFACE,
 					"PrimaryContexts",
 					DBUS_TYPE_OBJECT_PATH, &objpath_list);
 
@@ -1275,7 +1184,7 @@ static void gprs_deactivate_for_remove(const struct ofono_error *error,
 	char **objpath_list;
 
 	if (error->type != OFONO_ERROR_TYPE_NO_ERROR) {
-		ofono_debug("Removing context failed with error: %s",
+		DBG("Removing context failed with error: %s",
 				telephony_error_to_str(error));
 
 		__ofono_dbus_pending_reply(&gprs->pending,
@@ -1301,7 +1210,7 @@ static void gprs_deactivate_for_remove(const struct ofono_error *error,
 		DBusConnection *conn = ofono_dbus_get_connection();
 
 		ofono_dbus_signal_array_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
+					OFONO_DATA_CONNECTION_MANAGER_INTERFACE,
 					"PrimaryContexts",
 					DBUS_TYPE_OBJECT_PATH, &objpath_list);
 		g_strfreev(objpath_list);
@@ -1341,7 +1250,7 @@ static DBusMessage *gprs_remove_context(DBusConnection *conn,
 		storage_sync(gprs->imsi, SETTINGS_STORE, gprs->settings);
 	}
 
-	ofono_debug("Unregistering context: %s\n", ctx->path);
+	DBG("Unregistering context: %s\n", ctx->path);
 	context_dbus_unregister(ctx);
 	gprs->contexts = g_slist_remove(gprs->contexts, ctx);
 
@@ -1352,7 +1261,7 @@ static DBusMessage *gprs_remove_context(DBusConnection *conn,
 	if (objpath_list) {
 		path = __ofono_atom_get_path(gprs->atom);
 		ofono_dbus_signal_array_property_changed(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
+					OFONO_DATA_CONNECTION_MANAGER_INTERFACE,
 					"PrimaryContexts",
 					DBUS_TYPE_OBJECT_PATH, &objpath_list);
 		g_strfreev(objpath_list);
@@ -1405,26 +1314,13 @@ void ofono_gprs_detached_notify(struct ofono_gprs *gprs)
 	 */
 }
 
-void ofono_gprs_status_notify(struct ofono_gprs *gprs,
-				int status, int lac, int ci, int tech)
+void ofono_gprs_status_notify(struct ofono_gprs *gprs, int status)
 {
 	/* If we are not attached and haven't tried to attach, ignore */
-	if (gprs->driver_attached == FALSE &&
-			(gprs->flags & GPRS_FLAG_ATTACHING) == 0)
+	if (gprs->driver_attached == FALSE)
 		return;
 
-	if (gprs->status != status)
-		set_registration_status(gprs, status);
-
-	if (gprs->location != lac)
-		set_registration_location(gprs, lac);
-
-	if (gprs->cellid != ci)
-		set_registration_cellid(gprs, ci);
-
-	if (gprs->technology != tech)
-		set_registration_technology(gprs, tech);
-
+	gprs->status = status;
 	gprs_attached_update(gprs);
 }
 
@@ -1484,7 +1380,7 @@ void ofono_gprs_context_deactivated(struct ofono_gprs_context *gc,
 
 		value = FALSE;
 		ofono_dbus_signal_property_changed(conn, ctx->path,
-						DATA_CONTEXT_INTERFACE,
+						OFONO_DATA_CONTEXT_INTERFACE,
 						"Active", DBUS_TYPE_BOOLEAN,
 						&value);
 	}
@@ -1639,9 +1535,10 @@ static void gprs_unregister(struct ofono_atom *atom)
 		gprs->netreg = NULL;
 	}
 
-	ofono_modem_remove_interface(modem, DATA_CONNECTION_MANAGER_INTERFACE);
+	ofono_modem_remove_interface(modem,
+					OFONO_DATA_CONNECTION_MANAGER_INTERFACE);
 	g_dbus_unregister_interface(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE);
+					OFONO_DATA_CONNECTION_MANAGER_INTERFACE);
 }
 
 static void gprs_remove(struct ofono_atom *atom)
@@ -1702,9 +1599,6 @@ struct ofono_gprs *ofono_gprs_create(struct ofono_modem *modem,
 
 	gprs->status = NETWORK_REGISTRATION_STATUS_UNKNOWN;
 	gprs->netreg_status = NETWORK_REGISTRATION_STATUS_UNKNOWN;
-	gprs->technology = -1;
-	gprs->cellid = -1;
-	gprs->location = -1;
 	gprs->pid_map = idmap_new(MAX_CONTEXTS);
 
 	return gprs;
@@ -1885,16 +1779,17 @@ void ofono_gprs_register(struct ofono_gprs *gprs)
 	struct ofono_atom *sim_atom;
 
 	if (!g_dbus_register_interface(conn, path,
-					DATA_CONNECTION_MANAGER_INTERFACE,
+					OFONO_DATA_CONNECTION_MANAGER_INTERFACE,
 					manager_methods, manager_signals, NULL,
 					gprs, NULL)) {
 		ofono_error("Could not create %s interface",
-				DATA_CONNECTION_MANAGER_INTERFACE);
+				OFONO_DATA_CONNECTION_MANAGER_INTERFACE);
 
 		return;
 	}
 
-	ofono_modem_add_interface(modem, DATA_CONNECTION_MANAGER_INTERFACE);
+	ofono_modem_add_interface(modem,
+				OFONO_DATA_CONNECTION_MANAGER_INTERFACE);
 
 	sim_atom = __ofono_modem_find_atom(modem, OFONO_ATOM_TYPE_SIM);
 
