@@ -168,17 +168,33 @@ void __ofono_atom_free(struct ofono_atom *atom);
 #include <ofono/cbs.h>
 #include <ofono/devinfo.h>
 #include <ofono/phonebook.h>
-#include <ofono/sms.h>
 #include <ofono/voicecall.h>
 #include <ofono/gprs.h>
 #include <ofono/gprs-context.h>
 #include <ofono/radio-settings.h>
 
+#include <ofono/sms.h>
+
+struct sms;
+
+enum ofono_sms_submit_flag {
+	OFONO_SMS_SUBMIT_FLAG_REQUEST_SR =	0x1,
+	OFONO_SMS_SUBMIT_FLAG_RECORD_HISTORY =	0x2,
+	OFONO_SMS_SUBMIT_FLAG_RETRY =		0x4,
+};
+
+typedef void (*ofono_sms_txq_submit_cb_t)(gboolean ok, void *data);
+
+unsigned int __ofono_sms_txq_submit(struct ofono_sms *sms, GSList *list,
+					unsigned int flags,
+					ofono_sms_txq_submit_cb_t cb,
+					void *data, ofono_destroy_func destroy);
+
 #include <ofono/sim.h>
 #include <ofono/stk.h>
 
-void __ofono_cbs_sim_download(struct ofono_stk *stk,
-				const guint8 *pdu, int pdu_len);
+struct cbs;
+void __ofono_cbs_sim_download(struct ofono_stk *stk, const struct cbs *msg);
 
 #include <ofono/ssn.h>
 
