@@ -2,7 +2,7 @@
  *
  *  oFono - Open Source Telephony
  *
- *  Copyright (C) 2008-2009  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2008-2010  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -42,22 +42,31 @@ int create_dirs(const char *filename, const mode_t mode)
 	const char *prev, *next;
 	int err;
 
+	if (filename[0] != '/')
+		return -1;
+
 	err = stat(filename, &st);
 	if (!err && S_ISREG(st.st_mode))
 		return 0;
 
-	dir = g_malloc(strlen(filename) + 1);
+	dir = g_try_malloc(strlen(filename) + 1);
+	if (dir == NULL)
+		return -1;
+
 	strcpy(dir, "/");
 
-	for (prev = filename; (next = strchr(prev + 1, '/')); prev = next)
-		if (next > prev + 1) {
-			strncat(dir, prev + 1, next - prev);
+	for (prev = filename; (next = strchr(prev + 1, '/')); prev = next) {
+		/* Skip consecutive '/' characters */
+		if (next - prev == 1)
+			continue;
 
-			if (mkdir(dir, mode) && errno != EEXIST) {
-				g_free(dir);
-				return -1;
-			}
+		strncat(dir, prev + 1, next - prev);
+
+		if (mkdir(dir, mode) == -1 && errno != EEXIST) {
+			g_free(dir);
+			return -1;
 		}
+	}
 
 	g_free(dir);
 	return 0;
